// kinc_companion
/*
	Helper functions for companions.
*/
// ChazM 4/14/05
// BMA 5/5/05 added GetCompInfluenceGlobVar()
// ChazM 6/8/05 added SetAssociatesState(), SetAllAssociatesState(), and SetAllAssociatesFollow()
// ChazM 6/8/05 added ClearAssociatesState(), ClearAllAssociatesActions(), and modified SetAllAssociatesFollow()
// BMA-OEI 6/23/05 added influence case for Ammon Jerro
// ChazM 8/02/05 added IsHenchmanByTag(), added prototypes
// ChazM 8/03/05 modified IsHenchmanByTag(), added GetHenchmanByTag(), RemoveHenchmanByTag()
// ChazM 8/15/05 added Tag and resref constants for companions
// ChazM 8/16/05 added SpawnCompanions(), included ginc_misc
// BDF 9/22/05 added RemoveAllCompanions()
// ChazM 11/18/05  added AddCompanionsToRoster() and AddCompanionToRoster()
// ChazM 11/22/05  added GetInfluence(), GetCompanionTagByNumber(), modified GetCompInfluenceGlobVar()
// 				added ApplyHenchmanModifier(), RemoveHenchmanModifier(), AddHenchmanToCompanion(), RemoveHenchmanFromCompanion()
// ChazM 12/1/05 Removed ROSTER_* constants - w/ 20 char limit, we can now just use tag.  Moved in functions from ginc_roster
// ChazM 12/16/05 added bevil
// EPF 1/9/06 Updated RemoveAllCompanions to use roster companions instead of henchmen.  Added DespawnAllCompanions().
// EPF 1/28/06 Added GetIsInParty()
// BMA-OEI 2/6/06 Added DespawnAllRosterMembers(), GetIsInRoster(), GetIsObjectInParty(), delay to DespawnAllCompanions()
// EPF 3/1/06 -- Added a bDespawn parameter to RemoveAllCompanions().  Also switched out Ben's delay for a 
// 			 	 reset of the loop because DelayCommand() is the fifth horseman of the apocalypse. 
// BMA-OEI 4/10/06 -- Added RemoveRosterMembersFromParty(), GetNumRosterMembersInParty()
// BMA-OEI 4/12/06 -- Added SaveRosterLoadModule()
// DBR 4/27/6 - moved GetNumHenchman() to ginc_henchman
// BMA-OEI 5/19/06 -- AddCompanionsToRoster() sets CampaignNPC=1
// BMA-OEI 7/10/06 -- Added GetInfluenceByNumber(), IncInfluenceByNumber(), SetInfluenceByNumber(), companion influence cap vars
// BMA-OEI 8/16/06 -- Added ForceRestParty()
// ChazM 8/24/06 - split from ginc_companions - kept comments in this file.
// ChazM 9/20/06 - ginc_object, ginc_henchman added - directly dependent

// ARH 12/19/09 - GetInfluenceAdjustment() for setting initial influence when first meeting companion.

// void main(){}

#include "ginc_henchman"
#include "ginc_companion"
#include "ginc_object"

const string RR_AMMON 		= "co_ammon_jerro";
const string RR_BISHOP 		= "co_bishop";
const string RR_CASAVIR		= "co_casavir";
const string RR_CONSTRUCT	= "co_construct";
const string RR_ELANEE 		= "co_elanee";
const string RR_GROBNAR 	= "co_grobnar";
const string RR_KHELGAR 	= "co_khelgar";
const string RR_NEESHKA 	= "co_neeshka";
const string RR_QARA 		= "co_qara";
const string RR_SAND 		= "co_sand";
const string RR_SHANDRA 	= "co_shandra";
const string RR_ZHJAEVE 	= "co_zhjaeve";
const string RR_BEVIL 		= "npc_bevil";

// we'll always use tag as the roster name 
const string TAG_AMMON 		= "ammon_jerro";
const string TAG_BISHOP 	= "bishop";
const string TAG_CASAVIR	= "casavir";
const string TAG_CONSTRUCT	= "construct";
const string TAG_ELANEE 	= "elanee";
const string TAG_GROBNAR 	= "grobnar";
const string TAG_KHELGAR 	= "khelgar";
const string TAG_NEESHKA 	= "neeshka";
const string TAG_QARA 		= "qara";
const string TAG_SAND 		= "sand";
const string TAG_SHANDRA 	= "shandra";
const string TAG_ZHJAEVE 	= "zhjaeve";
const string TAG_BEVIL 		= "npc_bevil";	// minor companions

const int MAX_COMPANIONS	= 13;

// Prototypes
void AddCompanionsToRoster();
void SpawnCompanions();
string GetCompanionTagByNumber(int nCompanion);
int GetCompanionNumberByTag(string sTag);
string GetCompInfluenceGlobVar(int nCompanion);
int RemoveAllCompanions( object oPC , int bDespawn = TRUE);

// Influence functions by campaign companion number

// Return PC's influence of companion ID nCompanion
int GetInfluenceByNumber( int nCompanion );

// Increase PC's influence of companion by nInfluence
void IncInfluenceByNumber( int nCompanion, int nInfluence );

// Set PC's influence of companion to nInfluence
void SetInfluenceByNumber( int nCompanion, int nInfluence );


//-------------------------------------------------
// Function Definitions
//-------------------------------------------------


// adds all companions to the roster
// members in roster must still be added to party	
void AddCompanionsToRoster()
{
	AddCompanionToRoster( TAG_AMMON, TAG_AMMON, RR_AMMON );
	SetIsRosterMemberCampaignNPC( TAG_AMMON, TRUE );
	AddCompanionToRoster( TAG_BISHOP, TAG_BISHOP, RR_BISHOP );
	SetIsRosterMemberCampaignNPC( TAG_BISHOP, TRUE );
	AddCompanionToRoster( TAG_CASAVIR, TAG_CASAVIR, RR_CASAVIR );
	SetIsRosterMemberCampaignNPC( TAG_CASAVIR, TRUE );
	AddCompanionToRoster( TAG_CONSTRUCT, TAG_CONSTRUCT, RR_CONSTRUCT );
	SetIsRosterMemberCampaignNPC( TAG_CONSTRUCT, TRUE );
	AddCompanionToRoster( TAG_ELANEE, TAG_ELANEE, RR_ELANEE );
	SetIsRosterMemberCampaignNPC( TAG_ELANEE, TRUE );
	AddCompanionToRoster( TAG_GROBNAR, TAG_GROBNAR, RR_GROBNAR );
	SetIsRosterMemberCampaignNPC( TAG_GROBNAR, TRUE );
	AddCompanionToRoster( TAG_KHELGAR, TAG_KHELGAR, RR_KHELGAR );
	SetIsRosterMemberCampaignNPC( TAG_KHELGAR, TRUE );
	AddCompanionToRoster( TAG_NEESHKA, TAG_NEESHKA, RR_NEESHKA );
	SetIsRosterMemberCampaignNPC( TAG_NEESHKA, TRUE );
	AddCompanionToRoster( TAG_QARA, TAG_QARA, RR_QARA );
	SetIsRosterMemberCampaignNPC( TAG_QARA, TRUE );
	AddCompanionToRoster( TAG_SAND, TAG_SAND, RR_SAND );
	SetIsRosterMemberCampaignNPC( TAG_SAND, TRUE );
	AddCompanionToRoster( TAG_SHANDRA, TAG_SHANDRA, RR_SHANDRA );
	SetIsRosterMemberCampaignNPC( TAG_SHANDRA, TRUE );
	AddCompanionToRoster( TAG_ZHJAEVE, TAG_ZHJAEVE, RR_ZHJAEVE );
	SetIsRosterMemberCampaignNPC( TAG_ZHJAEVE, TRUE );
}


// spawn all companions
void SpawnCompanions()
{
	string WP_Prefix = "spawn_";

	if (GetIsObjectValid(GetObjectByTag(TAG_AMMON)) == FALSE)
		SpawnCreatureAtWP(RR_AMMON, 	WP_Prefix + TAG_AMMON);
	if (GetIsObjectValid(GetObjectByTag(TAG_BISHOP)) == FALSE)
		SpawnCreatureAtWP(RR_BISHOP, 	WP_Prefix + TAG_BISHOP);
	if (GetIsObjectValid(GetObjectByTag(TAG_CASAVIR)) == FALSE)
		SpawnCreatureAtWP(RR_CASAVIR, 	WP_Prefix + TAG_CASAVIR);
	if (GetIsObjectValid(GetObjectByTag(TAG_CONSTRUCT)) == FALSE)
		SpawnCreatureAtWP(RR_CONSTRUCT,	WP_Prefix + TAG_CONSTRUCT);
	if (GetIsObjectValid(GetObjectByTag(TAG_ELANEE)) == FALSE)
		SpawnCreatureAtWP(RR_ELANEE, 	WP_Prefix + TAG_ELANEE);
	if (GetIsObjectValid(GetObjectByTag(TAG_GROBNAR)) == FALSE)
		SpawnCreatureAtWP(RR_GROBNAR, 	WP_Prefix + TAG_GROBNAR);	
	if (GetIsObjectValid(GetObjectByTag(TAG_KHELGAR)) == FALSE)
		SpawnCreatureAtWP(RR_KHELGAR, 	WP_Prefix + TAG_KHELGAR);	
	if (GetIsObjectValid(GetObjectByTag(TAG_NEESHKA)) == FALSE)
		SpawnCreatureAtWP(RR_NEESHKA,	WP_Prefix + TAG_NEESHKA);	
	if (GetIsObjectValid(GetObjectByTag(TAG_QARA)) == FALSE)
		SpawnCreatureAtWP(RR_QARA, 		WP_Prefix + TAG_QARA);	
	if (GetIsObjectValid(GetObjectByTag(TAG_SAND)) == FALSE)
		SpawnCreatureAtWP(RR_SAND, 		WP_Prefix + TAG_SAND);		
	if (GetIsObjectValid(GetObjectByTag(TAG_SHANDRA)) == FALSE)
		SpawnCreatureAtWP(RR_SHANDRA, 	WP_Prefix + TAG_SHANDRA);
	if (GetIsObjectValid(GetObjectByTag(TAG_ZHJAEVE)) == FALSE)
		SpawnCreatureAtWP(RR_ZHJAEVE, 	WP_Prefix + TAG_ZHJAEVE);
}

//
string GetCompanionTagByNumber(int nCompanion)
{
	string sCompanionTag = "";

	switch (nCompanion)
	{
		case 1:
			sCompanionTag = TAG_KHELGAR;
			break;
		case 2:
			sCompanionTag = TAG_NEESHKA;
			break;
		case 3:
			sCompanionTag = TAG_ELANEE;
			break;
		case 4:
			sCompanionTag = TAG_QARA;
			break;
		case 5:
			sCompanionTag = TAG_SAND;
			break;
		case 6:
			sCompanionTag = TAG_GROBNAR;
			break;
		case 7:
			sCompanionTag = TAG_CASAVIR;
			break;
		case 8:
			sCompanionTag = TAG_BISHOP;
			break;
		case 9:
			sCompanionTag = TAG_SHANDRA;
			break;
		case 10:
			sCompanionTag = TAG_CONSTRUCT;
			break;
		case 11:
			sCompanionTag = TAG_ZHJAEVE;
			break;
		case 12:
			sCompanionTag = TAG_AMMON;
			break;
		case 13:
			sCompanionTag = TAG_BEVIL;
			break;
		default:
			PrintString("ginc_companions-GetCompanionTagByNumber: Invalid companion ID!"); 
			break;
	}
	return sCompanionTag;
}

// look through all the numbers for this Tag.
int GetCompanionNumberByTag(string sTag)
{
	string sCompareTag;
	int i;
	for (i=1; i<=MAX_COMPANIONS; i++)
	{
		sCompareTag = GetCompanionTagByNumber(i);
		if (sTag == sCompareTag)
			return(i);
	}
	return (0);
}

// Used in ga/gc_influence* scripts
string GetCompInfluenceGlobVar( int nCompanion )
{
	string sCompanionTag 	= GetCompanionTagByNumber(nCompanion);
	string sVarName			= INFLUENCE_PREFIX + sCompanionTag;
	return sVarName;
}

// Success should return 0, or remaining number of companions on failure
int RemoveAllCompanions( object oPC , int bDespawn = TRUE)
{
	RemoveRosterMemberFromParty( TAG_AMMON, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_BISHOP, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_CASAVIR, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_CONSTRUCT, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_ELANEE, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_GROBNAR, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_KHELGAR, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_NEESHKA, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_QARA, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_SAND, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_SHANDRA, oPC, bDespawn );
	RemoveRosterMemberFromParty( TAG_ZHJAEVE, oPC, bDespawn );
	
	return GetNumHenchmen( oPC );
}



// Return PC's influence of companion by CompanionID
int GetInfluenceByNumber( int nCompanion )
{
	string sVar = GetCompInfluenceGlobVar( nCompanion );
	int nRet = GetGlobalInt( sVar );
	return ( nRet );	
}

// Increase PC's influence of companion by nInfluence ( capped )
void IncInfluenceByNumber( int nCompanion, int nInfluence )
{
	nInfluence = nInfluence + GetInfluenceByNumber( nCompanion );
	SetInfluenceByNumber( nCompanion, nInfluence );
}

// Set PC's influence of companion to nInfluence ( capped )
void SetInfluenceByNumber( int nCompanion, int nInfluence )
{
	string sVar = GetCompInfluenceGlobVar( nCompanion );
	if ( nInfluence < INFLUENCE_MIN )
		nInfluence = INFLUENCE_MIN;
	else if ( nInfluence > INFLUENCE_MAX )
		nInfluence = INFLUENCE_MAX;
		
	SetGlobalInt( sVar, nInfluence );
	
}

// Gets initial influence for new companions based on PC's Charisma and relative alignment
int GetInfluenceAdjustment(object oPC, object oNPC)
{

	// Gets CH bonus based on raw Charisma, adjusted by race but not items, magic or other effects.
	int nAdjust = GetAbilityScore(oPC, ABILITY_CHARISMA, TRUE);
	int nRace = GetSubRace(oPC);
	if (nRace == RACIAL_SUBTYPE_GRAY_DWARF || nRace == RACIAL_SUBTYPE_SVIRFNEBLIN)
		nAdjust -= 4;
	else if (nRace == RACIAL_SUBTYPE_SHIELD_DWARF || nRace == RACIAL_SUBTYPE_HALFORC || nRace == RACIAL_SUBTYPE_GRAYORC || nRace == RACIAL_SUBTYPE_TIEFLING || nRace == RACIAL_SUBTYPE_AIR_GENASI || nRace == RACIAL_SUBTYPE_EARTH_GENASI || nRace == RACIAL_SUBTYPE_FIRE_GENASI || nRace == RACIAL_SUBTYPE_WATER_GENASI)
		nAdjust -= 2;
	else if (nRace == RACIAL_SUBTYPE_DROW || nRace == RACIAL_SUBTYPE_AASIMAR || nRace == RACIAL_SUBTYPE_YUANTI)
		nAdjust += 2;
	nAdjust = nAdjust / 2 - 5;

	// Alignment influence adjustment for each axis - non-neutral alignments only
	if (GetAlignmentGoodEvil(oPC) != ALIGNMENT_NEUTRAL)
	{
		if (GetAlignmentGoodEvil(oPC) == GetAlignmentGoodEvil(oNPC))
		{
			nAdjust += 2;
		}
		else if (GetAlignmentGoodEvil(oNPC) != ALIGNMENT_NEUTRAL && GetAlignmentGoodEvil(oPC) != GetAlignmentGoodEvil(oNPC))
		{
			nAdjust -= 2;
		}
	}
	if (GetAlignmentLawChaos(oPC) != ALIGNMENT_NEUTRAL)
	{
		if (GetAlignmentLawChaos(oPC) == GetAlignmentLawChaos(oNPC))
		{
			nAdjust += 2;
		}
		else if (GetAlignmentLawChaos(oNPC) != ALIGNMENT_NEUTRAL && GetAlignmentLawChaos(oPC) != GetAlignmentLawChaos(oNPC))
		{
			nAdjust -= 2;
		}
	}
	return (nAdjust);
}